#!/usr/bin/env python
# coding: utf-8
import logging
import os
import socket

__version__ = '0.1.0a1'

log = logging.getLogger(__name__)


class GrimReaper(object):
    def __init__(self, socket_path='/tmp/grimreaper.socket', process_timeout=30):
        self._path = socket_path
        self._socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self._process_timeout = process_timeout
        self._connect()

    def _is_connected(self):
        try:
            self._socket.recv(0)
        except socket.error:
            return self._connect()
        else:
            return True

    def _connect(self):
        try:
            self._socket.connect(self._path)
        except socket.error as e:
            log.error('Unnable to connect to the socket "%s": %s', self._path, e)
            self.on_connection_error(e)

            if e.errno == socket.errno.ENOENT:
                # No such file or directory
                return False
            elif e.errno == socket.errno.ECONNREFUSED:
                # Connection refused
                return False

            raise
        return True

    def register(self, timeout=None, pid=None):
        if timeout is None:
            timeout = self._process_timeout

        if pid is None:
            pid = os.getpid()

        if self._is_connected():
            msg = 'register:%s:%s' % (pid, timeout)
            self._socket.sendall(msg.encode('utf-8'))
            log.debug('Registered process (PID=%s; timeout=%s)', pid, timeout)
        else:
            log.warning('Unnable to register the process (PID=%s).', pid)

    def unregister(self, pid=None):
        if pid is None:
            pid = os.getpid()

        if self._is_connected():
            msg = 'unregister:%s' % pid
            self._socket.sendall(msg.encode('utf-8'))
            log.debug('Unregistered process (PID=%s)', pid)
        else:
            log.warning('Unnable to unregister the process (PID=%s).', pid)

    def on_connection_error(self, exc):
        pass



#!/usr/bin/env python

import re
from codecs import open

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def get_version():
    version = ''
    with open('grimreaper.py', 'r') as fd:
        version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                            fd.read(), re.MULTILINE).group(1)

    if not version:
        raise RuntimeError("Cannot find version's information")
    return version


def get_long_description():
    with open('README.rst', 'r', 'utf-8') as f:
        readme = f.read()

    with open('CHANGES.rst', 'r', 'utf-8') as f:
        changes = f.read()

    return readme + '\n\n' + changes

setup(
    name='GrimReapersPie',
    version=get_version(),
    description='Python client to the GrimReaper process killer.',
    long_description=get_long_description(),
    author='Mateusz Pawlik',
    author_email='matee+grimreaperspie@matee.net',
    url='https://github.com/matee911/GrimReapersPie',
    py_modules=['grimreaper'],
    license='Apache 2.0',
    keywords='management',
    classifiers=(
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ),
)
